#!/bin/bash

OS=`uname -s 2>/dev/null`
PLATFORM=`uname -m 2>/dev/null`
UDEV_DIR="/etc/udev"
UDEV_CONF_FILE="$UDEV_DIR/udev.conf"
LD=/lib/ld-linux.so.2
LD_x86_64=/lib64/ld-linux-x86-64.so.2
PLATFORMS="i386 i486 i586 i686 x86_64"
LDS="${LD} ${LD} ${LD} ${LD} ${LD_x86_64}"
API=wrg315api.so
DIR=$(pwd)

echo ""
echo "WiNRADiO G315 driver and API installation"
echo ""

if [ $(id -ur) != 0 ]; then
    echo "Please, run this installer as root."
    echo "Installation aborted."
    echo ""
    exit 1
fi

if [ "$OS" != "Linux" ]; then
    echo "Your operating system is not supported."
    echo "Installation aborted."
    echo ""
    exit 1
fi

SUPPORTED=0

for i in ${PLATFORMS} ; do
if [ "$i" == "$PLATFORM" ] ; then
    SUPPORTED=1
    break;
fi
done

if [ "$SUPPORTED" == "0" ] ; then
echo -e "Your hardware platform is not supported."
echo "Supported hardware platforms: $PLATFORMS"
echo "Detected hardware platform: $PLATFORM"
echo
echo "Press ENTER to exit."
read
exit 0  
fi

echo "Compiling G315 kernel module"

cd "driver"

if [ $? != 0 ]; then
    echo "Could not find G315 driver source code."
    echo "Installation aborted."
    echo ""
    exit 1
fi

make clean
make

echo ""

if [ ! -f "wrg315.ko" ]; then
    cd ..
    echo "Compilation of G315 kernel module failed."
    echo "Please, check if development tools and development package"
    echo "for building kernel modules are installed and check if your kernel"
    echo "supports loadable modules."
    echo "Installation aborted."
    echo ""
    exit 1
fi

echo "Writing udev rule"

if [ -f "$UDEV_CONF_FILE" ]; then
    udev_rules=""
    
    source "$UDEV_CONF_FILE"
    
    if [ "$udev_rules" != "" ]; then
	WiNRADiO_RULES="$udev_rules/WiNRADiO.rules"
    else
	WiNRADiO_RULES="$UDEV_DIR/rules.d/WiNRADiO.rules"
    fi
    
    if [ -f "$WiNRADiO_RULES" ]; then
	WRG3=`less "$WiNRADiO_RULES" | grep wrg3`
	
	if [ "$WRG3" == "" ]; then
	    if [ -s "$WiNRADiO_RULES" ]; then
    		echo "" >> "$WiNRADiO_RULES"
    	    fi
	
    	    echo "# WiNRADiO G3xx devices" >> "$WiNRADiO_RULES"
	    echo "KERNEL==\"wrg3*\"				MODE=\"0644\"" >> "$WiNRADiO_RULES"
	else
	    echo "G315 udev rule exists."	    
	fi
    else
	echo "# WiNRADiO G3xx devices" > "$WiNRADiO_RULES"
	echo "KERNEL==\"wrg3*\"				MODE=\"0644\"" >> "$WiNRADiO_RULES"
    fi
else
    echo "Warning: unable to find udev configuration file."
    echo "udev is not installed or its configuration file"
    echo "is another than $UDEV_CONF_FILE."
    echo "G315 udev rule will not be written."
    echo "It is possible only root will have the accces to"
    echo "the G315 devices."
    echo ""
fi

echo "Installing G315 kernel module"
make uninstall >/dev/null 2>&1
make install

echo ""

if [ $? != 0 ]; then
    cd ..

    echo "G315 kernel module installation failed."
    echo "Installation aborted."
    echo ""
    exit 1
fi

cd ..

/sbin/modprobe wrg315

echo "Installing G315 API"

if [ -f "/usr/lib/$API" ]; then
  rm -f "/usr/lib/$API"
fi

if [ -f "/usr/lib64/$API" ]; then
  rm -f "/usr/lib64/$API"
fi

for p in $PLATFORMS ; do
    f="lib/$p/$API"

    if [[ -f "$f" ]] ; then
    printf "%-8s" "$p:"
    status="Not supported"
    lib_dir=
    for l in $LDS ; do
        if test -x $l ; then
            $l --verify "$f" >/dev/null 2>&1
            r=$?
            case $r in
                0|2)
                    status="Failed"

                    lib_dir=$($l --list "$f" 2>/dev/null | grep /libc.so | sed 's/.*\s\//\//g' | sed 's/\/libc.*//g')

                    if [ "$lib_dir" != "" ] ; then
                        if test -f "$lib_dir/$API" ; then                                 
                                rm -f "$lib_dir/$API" >/dev/null 2>&1
                        fi
                        install -m 755 "$f" "$lib_dir/$API" >/dev/null 2>&1

                        if [ "$?" == "0" ] ; then
                            cd "$lib_dir"                                
                            cd "$DIR"
                            status="OK"
                        fi
                    fi
                    break;
                ;;
                *)
                ;;
            esac
        fi
    done

    printf "%-40s %15s\n" "$lib_dir/$API" "[$status]"
    fi
done

echo ""
echo "Done."
echo ""

exit 0
